@@ -9,6 +9,7 @@ from typing import Any, ClassVar, Dict, List, Optional, Union
 
 from isolate.backends import BaseEnvironment, EnvironmentCreationError
 from isolate.backends.common import (
+    active_python,
     get_executable_path,
     logged_io,
     sha256_digest_of,
@@ -23,6 +24,7 @@ class VirtualPythonEnvironment(BaseEnvironment[Path]):
 
     requirements: List[str] = field(default_factory=list)
     constraints_file: Optional[os.PathLike] = None
+    python_version: Optional[str] = None
 
     @classmethod
     def from_config(
@@ -41,7 +43,9 @@ class VirtualPythonEnvironment(BaseEnvironment[Path]):
                 constraints = stream.read().splitlines()
         else:
             constraints = []
-        return sha256_digest_of(*self.requirements, *constraints)
+
+        active_python_version = self.python_version or active_python()
+        return sha256_digest_of(active_python_version, *self.requirements, *constraints)
 
     def install_requirements(self, path: Path) -> None:
         """Install the requirements of this environment using 'pip' to the
@@ -73,31 +77,73 @@ class VirtualPythonEnvironment(BaseEnvironment[Path]):
             except subprocess.SubprocessError as exc:
                 raise EnvironmentCreationError("Failure during 'pip install'.") from exc
 
+    def _install_python_through_pyenv(self) -> str:
+        from isolate.backends.pyenv import PyenvEnvironment
+
+        self.log(
+            f"Requested Python version of {self.python_version} is not available "
+            "in the system, attempting to install it through pyenv."
+        )
+
+        pyenv = PyenvEnvironment.from_config(
+            {"python_version": self.python_version},
+            settings=self.settings,
+        )
+        return str(get_executable_path(pyenv.create(), "python"))
+
+    def _decide_python(self) -> str:
+        from virtualenv.discovery import builtin
+
+        from isolate.backends.pyenv import _get_pyenv_executable
+
+        interpreter = builtin.get_interpreter(self.python_version, ())
+        if interpreter is not None:
+            return interpreter.executable
+
+        try:
+            _get_pyenv_executable()
+        except Exception:
+            raise EnvironmentCreationError(
+                f"Python {self.python_version} is not available in your "
+                "system. Please install it first."
+            ) from None
+        else:
+            return self._install_python_through_pyenv()
+
     def create(self) -> Path:
         from virtualenv import cli_run
 
         venv_path = self.settings.cache_dir_for(self)
-        if venv_path.exists():
-            return venv_path
+        with self.settings.cache_lock_for(venv_path):
+            if venv_path.exists():
+                return venv_path
 
-        with self.settings.build_ctx_for(venv_path) as build_path:
-            self.log(f"Creating the environment at '{build_path}'")
+            self.log(f"Creating the environment at '{venv_path}'")
+
+            args = [str(venv_path)]
+            if self.python_version:
+                args.append(f"--python={self._decide_python()}")
 
             try:
-                cli_run([str(build_path)])
-            except OSError as exc:
+                cli_run(args)
+            except (RuntimeError, OSError) as exc:
                 raise EnvironmentCreationError(
-                    f"Failed to create the environment at '{build_path}'"
+                    f"Failed to create the environment at '{venv_path}'"
                 ) from exc
 
-            self.install_requirements(build_path)
+            self.install_requirements(venv_path)
 
-        assert venv_path.exists(), "Environment must be built at this point"
         self.log(f"New environment cached at '{venv_path}'")
         return venv_path
 
     def destroy(self, connection_key: Path) -> None:
-        shutil.rmtree(connection_key)
+        with self.settings.cache_lock_for(connection_key):
+            # It might be destroyed already (when we are awaiting
+            # for the lock to be released).
+            if not connection_key.exists():
+                return
+
+            shutil.rmtree(connection_key)
 
     def exists(self) -> bool:
         path = self.settings.cache_dir_for(self)
